use askama::Template;
use futures::future;

use gotham::handler::{HandlerFuture, IntoHandlerError};
use gotham::helpers::http::response::{create_empty_response, create_response};
use gotham::state::{FromState, State};
use gotham_derive::{StateData, StaticResponseExtender};

use hyper::StatusCode;
use mime;
use serde_derive::Deserialize;

use syntect::html::ClassedHTMLGenerator;

use crate::config::Config;
use crate::paste::Paste;
use crate::syntax::SYNTAX_SET;

#[derive(Deserialize, StateData, StaticResponseExtender)]
pub struct Params {
	id: String,
}

#[derive(Debug, Template)]
#[template(path = "view.html")]
pub struct View {
	id: String,
	dt: String,
	text_lines: Vec<String>,
	index_len: usize,
	site_url: String,
	syntax: String,
}

pub fn get(mut state: State) -> Box<HandlerFuture> {
	Box::new({
		let config = Config::take_from(&mut state);
		let Params { id } = Params::take_from(&mut state);

		let mut path = config.data_directory.clone();
		path.push(id.clone());

		match Paste::from_file(path) {
			Ok(paste) => {
				let text_lines: Vec<String> = paste.text.lines().map(|s| s.into()).collect();
				let index_len = format!("{}", text_lines.len()).len();
				let syntax = SYNTAX_SET
					.find_syntax_by_name(&paste.lang)
					.unwrap_or_else(|| SYNTAX_SET.find_syntax_plain_text());
				let mut high_lines: Vec<String> = Vec::new();

				for line in text_lines {
					let mut html_generator = ClassedHTMLGenerator::new(&syntax, &SYNTAX_SET);
					html_generator.parse_html_for_line(&line);
					high_lines.push(html_generator.finalize());
				}

				let template = View {
					id,
					dt: paste.dt.format("%Y-%m-%dT%H:%MZ").to_string(),
					text_lines: high_lines,
					index_len,
					site_url: config.url,
					syntax: paste.lang,
				};

				let res = match template.render() {
					Ok(content) => create_response(
						&state,
						StatusCode::OK,
						mime::TEXT_HTML_UTF_8,
						content.into_bytes(),
					),
					Err(_) => create_empty_response(&state, StatusCode::INTERNAL_SERVER_ERROR),
				};

				future::ok((state, res))
			}
			Err(e) => {
				let io_error = std::io::Error::new(std::io::ErrorKind::Other, e.description());
				future::err((state, io_error.into_handler_error()))
			}
		}
	})
}
